/***************************************************************************
                          devselector.cpp  -  description                              
                             -------------------                                         
    begin                : Fri Now 10 2000                                           
    copyright            : (C) 2000 by Klaas Freitag                         
    email                : freitag@suse.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qcstring.h>
#include <kapp.h>
#include <kconfig.h>

#include "resource.h"
#include "devselector.h"

/* Definitions for the kapplication Config object */
#define GROUP_STARTUP    "Startup"
#define STARTUP_SCANDEV  "ScanDevice"



DeviceSelector::DeviceSelector( QWidget *parent, QStrList& devList )
   :KDialogBase( parent,  "DeviceSel", true, I18N("Wellcome to Kooka"),
		 Ok, Ok, true )
{
   debug( "Starting DevSelector!" );
   // Layout-Boxes
   QWidget *page = new QWidget( this );
   CHECK_PTR( page );
   setMainWidget( page );

   QVBoxLayout *topLayout = new QVBoxLayout( page, marginHint(), spacingHint() );
   QLabel *label = new QLabel( page, "captionImage" );
   CHECK_PTR( label );
   label->setPixmap( *(new QPixmap( "kookalogo.png" )));
   label->resize( 100, 350 );
   topLayout->addWidget( label );

   selectBox = new QButtonGroup( 1, Horizontal, I18N( "Select a scan device" ),
				 page, "ButtonBox");
   CHECK_PTR( selectBox );
   selectBox->setExclusive( true );
   topLayout->addWidget( selectBox );
   setScanSources( devList );
   
#if 0
   QLabel *lpix = new QLabel( this );
   QPixmap *pix = new QPixmap( "bsa2.jpg" );
   QSize s = pix->size();
   lpix->resize(s);
   lpix->setPixmap( *pix );
   topLayout->addWidget( lpix );
#endif

}

QString DeviceSelector::getSelectedDevice( void ) const
{
   CHECK_PTR( selectBox );

   QButton *rb = selectBox->selected();

   if( rb )
   {
      return( rb->text() );
   }

   return( "" );
}
 

   
void DeviceSelector::setScanSources( QStrList& sources )
{
   bool default_ok = false;
   char *s = 0;
   
   kapp->config()->setGroup( GROUP_STARTUP );
   QString defstr = kapp->config()->readEntry( STARTUP_SCANDEV, "" );
   
   for ( s = sources.first(); selectBox && s; s=sources.next() )
   {
      QString css( s );
      
      QRadioButton *rb = new QRadioButton( css, selectBox );
      if( css == defstr )
      {
	 rb->setChecked( true );
	 default_ok = true;
      }
      selectBox->insert( rb );
   }

   /* Default still needs to be set */
   if( selectBox && ! default_ok )
   {
      /* if no default found, set the first */
      QRadioButton *rb = (QRadioButton*) selectBox->find( 0 );
      CHECK_PTR( rb );
      rb->setChecked( true );
   }
      
}

DeviceSelector::~DeviceSelector()
{
   
}

/* The End ;) */
